package es.uca.xmleye.gui.trees;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import org.w3c.dom.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;

/**
 * <p>
 * Dibujador de celdas basado en datos del árbol XML. Permite personalizar la
 * etiqueta y el icono.
 * </p>
 * 
 * <p>
 * Se asumen las siguientes cosas:
 * </p>
 * 
 * <ul>
 * <li>Se usa sobre un DOMTree, con modelo DOMTreeModel.</li>
 * <li>DOMTreeModel sólo muestra elementos.</li>
 * </ul>
 * 
 * <p>
 * La lógica es sencilla:
 * </p>
 * <ul>
 * <li>La etiqueta por defecto es el nombre del elemento.</li>
 * <li>El icono por defecto está determinado por el Look & Feel.</li>
 * <li>NODELABEL_ATTRIBUTE, si está presente, modifica la etiqueta.</li>
 * <li>NODEICON_ATTRIBUTE, una URL relativa al directorio de trabajo, si está
 * presente, modifica el icono.</li>
 * </ul>
 */
public class DataDrivenTreeCellRenderer extends DefaultTreeCellRenderer {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2448851844630408020L;
	/** Atributo que modifica la etiqueta. */
	public final static String NODELABEL_ATTRIBUTE = "nodelabel";
	/** Atributo que modifica el icono. */
	public final static String NODEICON_ATTRIBUTE = "nodeicon";

	public DataDrivenTreeCellRenderer() {
		super();
	}

	@Override
	public Component getTreeCellRendererComponent(JTree arbol, Object valor,
			boolean sel, boolean expandido, boolean hoja, int fila,
			boolean tieneFoco) {
		super.getTreeCellRendererComponent(arbol, valor, sel, expandido, hoja,
				fila, tieneFoco);

		final Element elem = (Element) valor;
		final String nombreTag = elem.getTagName();
		final String etiqNodo = elem.getAttribute(NODELABEL_ATTRIBUTE);
		final String iconoNodo = elem.getAttribute(NODEICON_ATTRIBUTE);

		setText(etiqNodo.length() > 0 ? etiqNodo : nombreTag);

		if (iconoNodo.length() > 0) {
			setIcon(new ImageIcon(iconoNodo));
		}

		return this;
	}

}
