package es.uca.xmleye.xpath;

/*
  Copyright (C) 2005-2008 Antonio García Domínguez
  
  This program is free software; you can redistribute it 
  and/or modify it under the terms of the GNU General Public 
  License as published by the Free Software Foundation; 
  either version 2 of the License, or (at your option) any 
  later version.
  
  This program is distributed in the hope that it will be 
  useful, but WITHOUT ANY WARRANTY; without even the implied 
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
  PURPOSE. See the GNU General Public License for more 
  details.
  
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 59 Temple Place, Suite 330, 
  Boston, MA 02111-1307 USA
*/

import junit.framework.*;
import junit.textui.*;
import javax.xml.parsers.*;
import java.io.*;
import org.w3c.dom.*;

import es.uca.xmleye.xpath.VectorNodeList;

import javax.xml.xpath.*;

/** 
    Pruebas de unidad sobre la clase VectorNodeList.
*/
public class VectorNodeListTest extends TestCase 
{
  private Document _docXML;
  private VectorNodeList listaVacia;
  private VectorNodeList lista1Elem;
  private VectorNodeList lista2Elems;
  private VectorNodeList lista3Elems;
  private VectorNodeList lista4Elems;
  private VectorNodeList lista5Elems;
  private VectorNodeList[] listasOriginales;
  private VectorNodeList[] listasInvertidas;

  /** 
      Crea una nueva colección de casos de prueba.
  */
  public VectorNodeListTest(String nombre) 
  {
    super(nombre);
  }

  /** 
      Crea el conjunto de casos de prueba a ejecutar mediante JUnit.
  */
  public static Test suite() 
  {
    return new TestSuite(VectorNodeListTest.class);
  }

  /** 
      Ejecuta mediante JUnit los casos de prueba.
   */
  public static void main(String[] args) 
  {
    TestRunner.run(suite());
  }

  /***** PRUEBAS ************************************/

  /** 
      Comprueba que realmente se invierten los elementos y que se mantienen.
  */
  public void testInversion() 
  {
    // Comparamos los elementos
    for (int i=0;i<listasInvertidas.length;++i) {
      assertTrue(listasOriginales[i].getLength() == listasInvertidas[i].getLength());
      final int len = listasOriginales[i].getLength();

      for (int j=0; j < len; ++j) {
	assertTrue(listasOriginales[i].item(j).equals(listasInvertidas[i].item(len-j-1)));
      }
    }
  }

  /** 
      Comprueba la idempotencia de la inversión.
  */
  public void testIdempotenciaInversion() 
  {
    for (int i=0;i<listasOriginales.length;++i) {
      final VectorNodeList copia = (VectorNodeList)listasOriginales[i].clone();
      copia.invert();
      copia.invert();
      assertTrue(copia.equals(listasOriginales[i]));
    }
  }


  /** 
      <p>Prepara los recursos necesarios para realizar las pruebas.</p>

      <p>Usando búsquedas XPath sobre un documento XML ya existente, se obtiene:</p>
      <ul>
       <li>Lista vacía.</li>
       <li>Listas con 1, 2, 3, 4 y 5 elementos.</li>
      </ul>
  */
  @Override
protected void setUp()
  {
    try {
      DocumentBuilder dBuilder = 
	DocumentBuilderFactory.newInstance().newDocumentBuilder();
      
      final FileInputStream fIS = 
	new FileInputStream("build/es/uca/xmleye/xpath/testXPath.xml");
      
      _docXML  = dBuilder.parse(fIS);

      /* Preparamos las listas */
      XPath xpath = XPathFactory.newInstance().newXPath();
      listaVacia  = new VectorNodeList();
      lista1Elem  = new VectorNodeList((NodeList)xpath.evaluate("defun[1]", _docXML, 
							       XPathConstants.NODESET));
      lista2Elems = new VectorNodeList((NodeList)xpath.evaluate("rule[position()<=1]", _docXML, 
							       XPathConstants.NODESET));
      lista3Elems = new VectorNodeList((NodeList)xpath.evaluate("rule[position()<=2]", _docXML, 
							       XPathConstants.NODESET));
      lista4Elems = new VectorNodeList((NodeList)xpath.evaluate("rule[position()<=3]", _docXML, 
							       XPathConstants.NODESET));
      lista5Elems = new VectorNodeList((NodeList)xpath.evaluate("rule[position()<=4]", _docXML, 
							       XPathConstants.NODESET));

      // Creamos listas para iterar sobre ellas en las pruebas
      listasOriginales = new VectorNodeList[] {
	listaVacia, lista1Elem, lista2Elems, 
	lista3Elems, lista4Elems, lista5Elems
      };
      
      // Creamos copias invertidas
      listasInvertidas = new VectorNodeList[] {
	(VectorNodeList)listaVacia.clone(), 
	(VectorNodeList)lista1Elem.clone(), 
	(VectorNodeList)lista2Elems.clone(), 
	(VectorNodeList)lista3Elems.clone(), 
	(VectorNodeList)lista4Elems.clone(), 
	(VectorNodeList)lista5Elems.clone()
      };
      for (int i=0;i<listasInvertidas.length;++i) {
	listasInvertidas[i].invert();
      }
      
    } catch (Exception ex) {
      fail(ex.getLocalizedMessage());
    }
  }

  /** Liberación de recursos usados para realizar las pruebas. */
  @Override
protected void tearDown() 
  {
    _docXML = null;
  }

}
